import sys
import logging

from PyQt5.QtWidgets import QApplication
from PyQt5.QtCore import Qt

from .app import App
from .gui import GIWAXSMainWindow, UncaughtHook, DebugWindow


def run(logging_level: int = logging.ERROR):
    loggers = [logging.getLogger(name) for name in logging.root.manager.loggerDict]
    for logger in loggers:
        logger.setLevel(logging_level)

    exception_hook = UncaughtHook()
    QApplication.setAttribute(Qt.AA_EnableHighDpiScaling, True)
    qapp = QApplication(sys.argv)
    giwaxs_app = App()
    if logging_level <= logging.DEBUG:
        debug_window = DebugWindow()
    window = GIWAXSMainWindow()
    sys.exit(qapp.exec_())



import logging
from scipy import sparse
from scipy.sparse.linalg import spsolve
import numpy as np

logger = logging.getLogger(__name__)


class SingletonMeta(type):
    _instance = None

    def __call__(cls):
        if cls._instance is None:
            cls._instance = super().__call__()
        return cls._instance


class InternalError(Exception):
    pass


def baseline_correction(y: np.ndarray,
                        smoothness_param: float,
                        asymmetry_param: float,
                        max_niter: int = 1000) -> np.ndarray:
    z = np.zeros_like(y)
    if smoothness_param <= 0 or asymmetry_param <= 0:
        return z
    y_size = y.size
    laplacian = sparse.diags([1, -2, 1], [0, -1, -2], shape=(y_size, y_size - 2))
    laplacian_matrix = laplacian.dot(laplacian.transpose())

    w = np.ones(y_size)
    for i in range(max_niter):
        W = sparse.spdiags(w, 0, y_size, y_size)
        Z = W + smoothness_param * laplacian_matrix
        z = spsolve(Z, w * y)
        w_new = asymmetry_param * (y > z) + (1 - asymmetry_param) * (y < z)
        if np.allclose(w, w_new):
            break
        w = w_new
    else:
        logger.info(f'Solution has not converged, max number of iterations reached.')
    return np.nan_to_num(z)



# -*- coding: utf-8 -*-
from typing import List
from pathlib import Path

from PyQt5.QtWidgets import (QWidget, QPushButton, QHBoxLayout,
                             QVBoxLayout, QFileDialog, QLabel)
from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot

from .tools import center_widget, show_error, Icon


class InitWindow(QWidget):
    sigOpenProject = pyqtSignal(object)
    sigExit = pyqtSignal()
    sigWidgetClosed = pyqtSignal()

    def __init__(self, recent_projects: List[Path] = None):
        flags = Qt.WindowFlags()
        flags |= Qt.WindowStaysOnTopHint | Qt.FramelessWindowHint
        super().__init__(flags=flags)
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setWindowTitle('GIWAXS analysis')
        self.setWindowIcon(Icon('window_icon'))
        self.__init_ui(recent_projects)
        self.setFixedWidth(300)
        center_widget(self)
        self.show()

    def __init_ui(self, recent_projects: List[Path] or None):
        layout = QVBoxLayout(self)
        new_project = QPushButton('New project')
        new_project.clicked.connect(self._new_project_dialog)
        layout.addWidget(new_project)

        if recent_projects:
            layout.addWidget(QLabel('Recent projects'))
            for path in recent_projects:
                btn = QPushButton(path.name)
                btn.clicked.connect(
                    lambda *x, p=path: self.sigOpenProject.emit(p))
                layout.addWidget(btn)

        exit_button = QPushButton('Exit')
        exit_button.clicked.connect(self.sigExit)
        layout.addWidget(QLabel(''))
        layout.addWidget(exit_button)

    @pyqtSlot(name='NewProjectDialog')
    def _new_project_dialog(self):
        folder = QFileDialog.getExistingDirectory(self, 'New project folder')
        if not folder:
            return
        folder = Path(folder)
        if folder.is_dir():
            self.sigOpenProject.emit(folder)
        else:
            show_error('Invalid folder. Please, select existing folder',
                       'Wrong folder')



from setuptools import setup, find_packages
from pathlib import Path


def read(filename: str):
    with open(Path(__file__).parent / filename, mode='r', encoding='utf-8') as f:
        return f.read()


setup(
        name='giwaxs_gui',
        packages=find_packages(),
        version='0.1.2.1',
        author='Vladimir Starostin',
        author_email='vladimir.starostin@uni-tuebingen.de',
        description='A GUI program for GIWAXS images analysis',
        long_description=read('README.md'),
        long_description_content_type='text/markdown',
        license='GPLv3',
        python_requires='>=3.7.2',
        entry_points={
                "gui_scripts": [
                    "giwaxs_gui = giwaxs_gui.__main__:run",
                ]
            },
        install_requires=[
            'numpy>=1.18.1',
            'opencv-python>=4.0.0.0',
            'scipy>=1.4.1',
            'h5py>=2.10.0',
            'PyQt5',
            'pyqtgraph==0.11.0rc0',
            'read_edf',
            'Pillow',
            'qdarkstyle',
            'qdarkgraystyle'
            ],
        include_package_data=True,
        keywords='xray python giwaxs scientific-analysis',
        url='https://pypi.org/project/giwaxs-gui',
)
